package com.ti.mobpo.ui

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.google.gson.annotations.SerializedName
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.launch
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import retrofit2.http.GET
import java.io.IOException

private const val BASE_URL = "https://pokeapi.co/api/v2/";
private const val SHOW_LIMIT = 20;
data class PokemonSpecies(
    @SerializedName("name") val name: String,
    @SerializedName("url") val url: String
)

data class PokemonResponse(
    @SerializedName("results") val results: List<PokemonSpecies>
)

interface PokeApiService {
    @GET("pokemon/?offset=0&limit=1025")
    suspend fun getPokemon(): PokemonResponse
}

class PokeSearchViewModel : ViewModel() {
    private val retrofit = Retrofit.Builder()
        .baseUrl(BASE_URL)
        .addConverterFactory(GsonConverterFactory.create())
        .build()

    private val service = retrofit.create(PokeApiService::class.java)

    private val _initialPokemonList = MutableStateFlow<List<PokemonSpecies>?>(null)
    private val _filteredPokemonList = MutableStateFlow<List<PokemonSpecies>?>(null)

    val pokemonList: StateFlow<List<PokemonSpecies>?> = _filteredPokemonList.asStateFlow()

    init {
        fetchPokemonSpecies()
    }

    private fun fetchPokemonSpecies() {
        viewModelScope.launch {
            try {
                val response = service.getPokemon()
                _initialPokemonList.value = response.results
                _filteredPokemonList.value = listOf(PokemonSpecies("Please enter a search term", ""));
            } catch (e: IOException) {
                /*TODO*/
            }
        }
    }

    fun search(query: String) {
        val initialPokemonList = _initialPokemonList.value
        if (query.isNotBlank() && initialPokemonList != null) {
            val filteredList = initialPokemonList.filter { it.name.contains(query, ignoreCase = true) }
            _filteredPokemonList.value = filteredList.take(SHOW_LIMIT);
        } else {
            _filteredPokemonList.value = listOf(PokemonSpecies("Please enter a search term", ""));
        }
    }
}